"""
See family.py for the precise documentation on this dataset, and for
the notation.

Briefly, we create the modular arithmetic dataset and its higher order
relations. 
"""

from matplotlib import mlab
from pylab import concatenate, array, arange, newaxis


# the complexity of the whole thing is supre cubic in the MOD. So MOD
# 20 (or 19) will already be too much to bear.
MOD = 12


### STEP 1:

A = range(MOD)
R_op = range(MOD) 

R_plus = range(MOD)
R_times = MOD + arange(MOD)


def gcd(a, b):
    assert(a >= 0 and b >= 0)
    if a == 0:
        return b
    elif b == 0: 
        return a
    elif a >= b:
        return gcd(b, a % b)
    elif b > a:
        return gcd(b, a)
    else:
        raise Exception ("gcd: hell broke loose!")

def phi(MOD):
    """
    number of numbers < MOD that are comprime to MOD.
    """
    return len([i for i in range(MOD) if gcd(i, MOD)==1])

phi_MOD = phi(MOD) 


# it is 



PLUS = [[a, r_id, (a + r) % MOD] 
        for a in A 
        for (r, r_id) in zip(R_op, R_plus)]

TIMES = [[a, r_id, (a * r) % MOD]
         for a in A 
         for (r, r_id) in zip(R_op, R_times)]



from pylab import concatenate
O_R0_O = concatenate((PLUS, TIMES))


### Now we wish to set up the meta relations. 
### The functions below essentially implement them;
### given a number, i, they return the id of the relation that does
### what the number i describes.  E.g., minus_i(i)-> returns the id of
### the operation plus_{MOD-i}.

def plus_i(i):
    return i % MOD

def minus_i(i):
    return (MOD - i) % MOD

def times_i(i):
    return MOD + i % MOD

def div_i(r):
    assert(gcd(r, MOD)==1)
    r_div = 1
    for i in range(phi_MOD - 1):
        r_div *= r
        r_div %= MOD
    return MOD + r_div




PLUS_META_id = 0
MINUS_META_id = 1
TIMES_META_id = 2
DIV_META_id = 3

## Note: the relations below are of type O_R1_R0.
PLUS_META = [[a, PLUS_META_id, plus_i(a)] 
             for a in range(MOD)]

MINUS_META = [[a, MINUS_META_id, minus_i(a)] 
              for a in range(MOD)]

TIMES_META = [[a, TIMES_META_id, times_i(a)] 
              for a in range(MOD)]

DIV_META = [[a, DIV_META_id, div_i(a)] 
            for a in range(MOD) if gcd(a, MOD)==1]

O_R1_R0 = concatenate((PLUS_META,
                       MINUS_META,
                       TIMES_META,
                       DIV_META))

## finally, we implement the -SINGLE- relation inv, which implements the
## inverse, and is of type R0_R2_R0.

INV_id = 0

PLUS_INV = [[r_id, INV_id, minus_i(r)] 
            for (r, r_id) in zip(R_op, R_plus)]

TIMES_INV = [[r_id, INV_id, div_i(r)] 
             for (r, r_id) in zip(R_op, R_plus)
             if gcd(r, MOD)==1]

R0_R2_R0 = concatenate((PLUS_INV, TIMES_INV))


NUM_O = MOD
NUM_R0 = MOD * 2 # number of basic relations: just plus and times
NUM_R1 = 4 # number of the higher order relations.
NUM_R2 = 1
