"""
See family.py for the precise documentation on this dataset, and for
the notation.

Briefly, we create the modular arithmetic dataset and its higher order
relations. 

But we do not use multiplication here. The reason is that, sadly,
multiplication simply does not so well when we try to predict it with
meta relations. In the paper we don't report results with
multiplication.

Note that unlike arith.py, we will not need any of the number
theoretic stuff like gcd or phi because adding and subtracting mod k
is easy.
"""

from matplotlib import mlab
from pylab import concatenate, array, arange, newaxis


# the complexity of the whole thing is supre cubic in the MOD. So MOD
# 20 (or 19) will already be too much to bear.
MOD = 12


### STEP 1:

A = range(MOD)
R_op = range(MOD) 

R_plus = range(MOD)



PLUS = [[a, r_id, (a + r) % MOD] 
        for a in A 
        for (r, r_id) in zip(R_op, R_plus)]


from pylab import concatenate
O_R0_O = PLUS

### Now we wish to set up the meta relations. 
### The functions below essentially implement them;
### given a number, i, they return the id of the relation that does
### what the number i describes.  E.g., minus_i(i)-> returns the id of
### the operation plus_{MOD-i}.

def plus_i(i):
    return i % MOD

def minus_i(i):
    return (MOD - i) % MOD

PLUS_META_id = 0
MINUS_META_id = 1

## Note: the relations below are of type O_R1_R0.
PLUS_META = [[a, PLUS_META_id, plus_i(a)] 
             for a in range(MOD)]

MINUS_META = [[a, MINUS_META_id, minus_i(a)] 
              for a in range(MOD)]

O_R1_R0 = concatenate((PLUS_META,
                       MINUS_META))

## finally, we implement the relation inv, which implements the
## inverse, and is of type R0_R2_R0.

INV_id = 0

PLUS_INV = [[r_id, INV_id, minus_i(r)] 
            for (r, r_id) in zip(R_op, R_plus)]

R0_R2_R0 = PLUS_INV # <-- avoid concatenate on a single thing.


NUM_O = MOD
NUM_R0 = MOD  # number of basic relations
NUM_R1 = 2
NUM_R2 = 1
