"""
MRE models relations and objects. 

We first describe the dataset of MRE.

There are four types of entities, which have names that are written in brackets:

- objects: O.
- basic relations: R0.
- meta relations R1: their first argument is an object (O) and their
- second argument is a basic relation (R1).
and 
- meta relations: R2: both of their arguments are basic relations (R0).

We will always use this notation for the four types of objects:
    O, R0, R1, R2.

================================================================
================================================================
================================================================

This is a data file that should be used for MRE.
It must import 4 files:

O_R0_O of dimensions (num data points)*3.

This is an array of instances of basic relations.
Each row is a triple of (object_id_1, basic_relation_id, object_id_2)
which is interpreted as (object_id_1, object_id_2) in basic_relation_id


O_R1_R0 of dimensions (num data points)*3

This is an array of instances of meta relations of type R1 -- that
takes an object and a basic relation.
Each row is a triple of (object_id_1, meta_R1_relation_id, basic_relation_id)
which is interpreted as (object_id_1, basic_relation_id) in meta_R1_relation_id


R0_R2_R0 of dimensions (num data points)*3

This si an array of instances of meta relations of type R2 -- that
take pairs of basic relations.
Each row is a triple of (basic_relation_id_1, meta_R2_relation_id, basic_relation_id_2)
which is interpreted as (basic_relation_id_1, basic_relation_id_2) in meta_R2_relation_id

In addition,
0 <= object_id < NUM_O
0 <= basic_relation_id < NUM_R0

0 <= meta_R1_relation_id < NUM_R1
0 <= meta_R2_relation_id < NUM_R2

The four constants above determine the number of objects and the
number of relations of each type.

This particular file imports the family tree dataset.

See also the file arith.py for the way in which it imports the
arithmetic data.


================================================================

Thus, to create your own data set, make sure to import the 7 variables
(for the datasets and the number of entities of each type).
"""

family_data_path =  'p8/mre/data/family_orig_data'

from matplotlib import mlab

def get_raw_family():
    """
    Read the text files and put them into nice formats.
    """
    NUM_PERSONS = 24
    NUM_RELATIONS = 12

    raw_family = mlab.load(family_data_path)
    
    person_A = raw_family[:, :NUM_PERSONS]
    relation = raw_family[:, NUM_PERSONS: NUM_PERSONS + NUM_RELATIONS]
    person_B = raw_family[:, -NUM_PERSONS:]

    assert((person_A.sum(1)==1).all())
    assert((relation.sum(1)==1).all())
    assert((person_B.sum(1)==1).all())

    return person_A, relation, person_B


from pylab import amap, find, concatenate, empty
def get_all_family():
    """
    produce the training and the test set for the relation.
    """
    person_A, relation, person_B = get_raw_family()

    person_A_id = amap(find, person_A)
    relation_id = amap(find, relation)
    person_B_id = amap(find, person_B)

    basic_rels = concatenate((person_A_id, relation_id, person_B_id), 1)
    return basic_rels



INT_TO_REL= {0 : 'has_wife',
             1 : 'has_husband',
             2 : 'has_son',
             3 : 'has_daughter',
             4 : 'has_father',
             5 : 'has_mother',
             6 : 'has_brother',
             7 : 'has_sister',
             8 : 'has_nephew',
             9 : 'has_niece',
             10: 'has_uncle',
             11: 'has_aunt' }
# we also have 'higher_oppsex' as the sole relation in meta_rel_rel.
# we've got nothing in meta_obj_rel.


################################################################
################################################################
################################################################
## So in the end: to make your own files, produce arrays with the
## following names:


O_R0_O = get_all_family()
O_R1_R0 = empty((0, 3))  # 
from pylab import array
R0_R2_R0  = array([[0, 0, 1],  # This is the relation higher_oppsex.
                   [1, 0, 0],  # Note that it is of type R2, as both
                   [2, 0, 3],  # of its operands are basic relations.
                   [3, 0, 2],
                   [4, 0, 5], 
                   [5, 0, 4],
                   [6, 0, 7],
                   [7, 0, 6],
                   [8, 0, 9],
                   [9, 0, 8],
                   [10,0,11],
                   [11,0,10]])


### The numbers of objects of each type.
NUM_O = 24
NUM_R0 = 12
NUM_R1 = 0
NUM_R2 = 1
