"""
This file is an example on how to set experiments for MRE. 
The file experiments does all the experiments and produce a very
similar array of numbers that appear in the table of the papers. 

We go over all the necessary steps, and the way in which the results
are obtained.

experiments.py is the main experiments file.

"""

# import basic useful stuff
from pylab import rand, ones, zeros
import p8.mre.mre as mre
import p8.mre.utils as utils
reload (mre)
import p8.mre.simulation_factory as simulation_factory
reload (simulation_factory)



# Use the family dataset. Any other data module could be used if it
# had all the variables listed in p8/mre/data/family.py
# Please read the file p8/mre/data/family.py. It describes our
# notation of the datasets (things like O, R0, R1, R2, and O_R0_O,
# O_R1_R0, R0_R2_R0) and their meanings. Reading it will make life easier.
import p8.mre.data.family as data
reload (data)


# In this point we choose the -basic relations- on which we'll train.# (Recall that the basic relations are held in the arrayO_R0_O). 
# There are two subroutines for doing that. The first,
# utils.random_train_indicies will select a random subset of a given
# length.
# The second knocks off all instances of a certain basic relation.
# The variable train_O_R0_O_inds is a boolean vector that represents
# the training set: if train_O_R0_O_inds[i]==True, then i is in the
# training set. 


### NOTE: we are holding out a large number of relations so that the 
### grad check will not fail. 
num_cases_to_hold_out = 100
train_O_R0_O_inds = utils.random_train_indicies(len(data.O_R0_O),
                                                len(data.O_R0_O) - num_cases_to_hold_out)
#train_O_R0_O_inds = utils.train_indicies_without_relation(data.O_R0_O,
#                                                          5)


# If we want, we can take the intersection  of the training sets by
# 
#train_O_R0_O_inds = utils.random_train_indicies(...) & utils.train_indicies_without_relation(...)
#
# 

# Which meta relations do we wish to use -for training-?
# 
# these are the object-basic-relation meta relations (of type R1)
train_O_R1_R0_inds = ones(len(data.O_R1_R0)).astype(bool)
#
#
# and these are the basic-relations basic-relations meta relations.
train_R0_R2_R0_inds = ones(len(data.R0_R2_R0)).astype(bool)
#
#
# In the lines of code above, they are set to
# ones(...).astype(bool). As usual, the set of relations to train on
# is represented as a vector of bools. By replacing the above "ones"
# with "zeros", we can remove the meta relations form the training and
# see the result on the algorithm's performance. 


### The dimensionality of the matrices.
dim = 4 



### Now we are making a loop of experiments.

NUM_EXPER = 5
print 'I will now run %s experiments on the same training/test split.\n' % NUM_EXPER
## the same train/test split because the train_*_inds were instansiated above.

for i in range(NUM_EXPER):
    # Create an MRE object. See mre/mre.py for precise details of its
    # parameters. Briefly, it gets all the training sets, all the
    # training indicies, the numbers of each objects, relations, and
    # meta relations.
    mre_object = mre.Mre(
        dim,
        
        # the numbers of different objects/relations
        data.NUM_O,
        data.NUM_R0,
        data.NUM_R1,
        data.NUM_R2,
                     
        # the datasets
        data.O_R0_O,
        data.O_R1_R0,
        data.R0_R2_R0,
                     
        # the training set
        train_O_R0_O_inds, 
        train_O_R1_R0_inds,
        train_R0_R2_R0_inds)




    print ('Running experiment %s.\n\
Train O_R0_O (basic relations) size = %s.\n\
Train O_R1_R0 meta rel size = %s.\n\
Train R0_R2_R0 meta rel size = %s.\n\
It will take a few minutes (maybe even 5, but not much more (on a 3.5GHz machine)).' 
           % (i, 
              sum(train_O_R0_O_inds),
              sum(train_O_R1_R0_inds),
              sum(train_R0_R2_R0_inds)))

    # We run MRE from some initial parameters and some weight decay.
    # see mre.simulation_factory for the precise details. Basically,
    # the mre_object is wrapped in some functions that are then given
    # to the BFGS solver.
    learned_mre = simulation_factory.run(mre_object, 
                                         check_grad=True)


    # learned_mre has the good parameters, so we can perform a
    # nearest-neighbor test.
    # (note: train_errors and test_errors are a pair.
    # The first part the pair is the number of error, and the
    # second part is the total number of cases).
    train_errors, test_errors = learned_mre.nearest_neighbor()


    print 'Train errors = %s / %s' % (train_errors[0],
                                      train_errors[1])

    print 'Test errors = %s / %s' % (test_errors[0],
                                     test_errors[1])



# If you are curious, you can look inside learned_mre by typing
# mre.experiments.learned_mre and look at its learned parameters and
# other variables.

