from scipy import optimize
import sys

scale = 1e-3
weight_decay = .1
def run(mre_shell, check_grad=False):

    def cost_fn(x):
        current_mre = mre_shell.unpack(x)
        
        cost =  scale * (current_mre.cost() + weight_decay * (x**2).sum()/2)

        return cost
        
    
    def grad_fn(x):
        current_mre = mre_shell.unpack(x)
        
        grad = scale * (current_mre.grad() + weight_decay * current_mre)

        return grad.flatten()

    def callback_fn(xk):
        current_mre = mre_shell.unpack(xk)
        
        cost = scale * current_mre.cost()

        print ('iteration, cost=%10.5f, |x|=%10.5f' % (cost, (xk**2).sum()))

    
    if check_grad:
        print 'performing a check_grad:'
        print optimize.check_grad(cost_fn, grad_fn, mre_shell.flatten())

    ## run the optimization procedure.
        
    sys.stdout = file('stdout','w')
    try:
        learned_mre_ = optimize.fmin_bfgs(cost_fn,

                                          mre_shell.flatten(),

                                          grad_fn,

                                          callback=callback_fn)

    except:
        # since we modify stdout for a while, we must undo the change
        # to make sure things will be printable later on.
        sys.stdout = sys.__stdout__
        raise 
    sys.stdout = sys.__stdout__
    
    
    learned_mre = mre_shell.unpack(learned_mre_)



    return learned_mre
